#!/usr/bin/python

import sys, os, os.path, re, string, optparse


page = string.Template("""
<html>
<body>
<title>CONFIG_$conf: $prompt</title>
</body>
<head>

<h1>CONFIG_$conf: $prompt (Linux kernel configuration option)</h1>

<h2>general</h2>

<p>$conf (CONFIG_$conf) is a Linux kernel configuration option of type '$type'.</p>

<h2>help text</h2>

<p>$help</p>

<h2>Other informations</h2>

<ul>
  <li>defined in: $kconf_filename</li>
<ul>

<h2>Hardware</h2>

<p>From <a href="http://cateee.net/lkddb/">lkddb</a>:</p>

<p>$lkddb</p>

<h2>Note</p2>

<p><b>Pages under construction</b>, so use with care!</p>
<p>These pages are automatic generated. Sources can be found in ...</p>

</head>
</html>
""")


# ---- parse lkddb ----

kconf_re = re.compile(r"^config\s+(\w+)\s*$(.*?)(?:^[a-z]|\Z)",
		re.MULTILINE | re.DOTALL)
lkddb_dict = {}

def parse_lkddb():
    "read lkddb and put in a CONFIG_ dict"
    f = open("lkddb.data")
    lkddb = f.readlines()
    f.close()
    for line in lkddb:
        for field in line.split():
	    if field == "CONFIG__UNKNOW__":
	        continue
	    if field.startswith("CONFIG_"):
	        conf = field[7:]
		if not lkddb_dict.has_key(conf):
	            lkddb_dict[conf] = [line]
		else:
		    lkddb_dict[conf].append(line)

# ---- parse Kconfig files ----

config = {}

def parse_kconf(src, filename):
    "read config menu in Kconfig"
    for conf, block in kconf_re.findall(src):
	if not config.has_key(conf):
	    config[conf] = { "_kconf": [filename] }
	else:
	    config[conf]["_kconf"].append(filename)
	
	lines = block.split("\n")
	max_item = len(lines)-1
	i = 0
	while(True):
	    if i > max_item:
	        break
	    line = lines[i].strip()
	    if line == "":
	        i+=1
		continue
	    elif line == "help"  or  line == "--help--":
		i+=1
		item = "help"
		value = ""
		help_ident = -1
	    	while(True):
                    if i > max_item:
                        break
		    line = lines[i].expandtabs()
		    if line.strip() == "":
		        value += "\n"
		    else:
		        sline = line.lstrip(" ")
		        ident = len(line) - len(sline)
		        if help_ident == -1:
		    	    help_ident = ident
		        elif ident < help_ident:
		            break
		        value += sline + "\n"
		    i +=1
	    else:
	        args = line.split()
		if len(args) == 1:
		    item = args[0]
		    value = None
		else:
		    try:
		        item = args[0]
			value = " ".join(args[1:])
		    except:
		        print "***", line, "***"
			print len(line.split())
			print line.split()
			raise
	    config[conf][item] = value
	    i+=1


parser = optparse.OptionParser()
parser.set_defaults(verbose=True, dbfile="lkddb.data")
parser.add_option("-q", "--quiet", dest="verbose",
                    action="store_false",
                    help="don't print warning messages")
parser.add_option("-o", "--output", dest="output",
                    action="store", type="string",
                    help="write warnings to FILE", metavar="FILE")
parser.add_option("-f", "--file", dest="dbfile",
                    action="store", type="string",
                    help="FILE to put the database", metavar="FILE")


(options, args) = parser.parse_args()

if options.output:
    logfile = open(options.output, "w")
else:
    logfile = sys.stdout

if len(args) < 1:
    parser.error("incorrect number of arguments")

kerneldir = args[0]
if kerneldir[-1] != "/":
    kerneldir += "/"
kerdir_len = len(kerneldir)

if len(args) > 1:
    dirs = args[1:]
else:
    dirs = ("arch", "block", "crypto", "drivers", "fs", "init",
                "ipc", "kernel", "lib", "mm", "net", "security", "sound")

print "doing the directory %s and the sub-dirs: %s" % (
 kerneldir, ", ".join(dirs) )

parse_lkddb()

for subdir in dirs:
    for root_full, d_, files in os.walk(os.path.join(kerneldir, subdir)):
        dir = root_full[kerdir_len:]
        for kconf in files:
            if not kconf.startswith("Kconfig"):
                continue
            filename = os.path.join(dir, kconf)
	    f = open(os.path.join(kerneldir, filename))
	    src = f.read()
	    f.close()
	    parse_kconf(src, filename)

# -------------------

def extract_key(dict, items):
    for item in items:
	if dict.has_key(item):
	    type = item
	    value = dict[item]
	    break
    else:
	return "", ""
    if value == None:
	value = ""
    return type, value


dir = "config/"

if not os.path.isdir(dir):
    print "could not find config/ Am I in the right dir?"
    sys.exit(1)

for conf, data in config.iteritems():
    print "Doing,", conf
    fn = os.path.join(dir, conf+".html")
    f = open(fn, "w")

    out = {"conf": conf.strip()}
    out["kconf_filename"] = data["_kconf"]

    type, value = extract_key(data, ("bool", "tristate", "string", "hex", "int"))
    if  value == ""  and  data.has_key("prompt"):
	value = data["prompt"]
    out["type"] = type
    out["prompt"] = value

    help = data.get("help", "(None)")
    out["help"] = help.strip().replace("\n\n", "</p>\n\n<p>")
    
    if lkddb_dict.has_key(conf):
	out["lkddb"] = "</p>\n<p>".join(lkddb_dict[conf])
    else:
	out["lkddb"] = "(none)"

    f.write(page.substitute(out))
    f.close()

